"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GsOTP = void 0;
const got_1 = require("got");
const GSOTP_HOST = 'https://api.gsotp.com';
class GsOTP {
    /**
     * @param apiKey Your API key in gsOTP.com.
     * @param manual Generate OTP code automatically or you gonna to enter it manually. <br>Default is `false`.
     * @param language Target language for error messages.
     */
    constructor(apiKey, manual = false, language = 'fa-IR') {
        this.apiKey = apiKey;
        this.language = language;
        this.sendSMS = this.sendSMS.bind(this);
        this.sendIVR = this.sendIVR.bind(this);
        this.sendGapMessage = this.sendGapMessage.bind(this);
        this.verify = this.verify.bind(this);
        this.getStatus = this.getStatus.bind(this);
    }
    request(path, body) {
        let language = this.language;
        if ('language' in body) {
            language = body.language;
            delete body.language;
        }
        return got_1.default(GSOTP_HOST + path, {
            headers: {
                'accept-language': language,
                'content-type': 'application/json',
                apiKey: this.apiKey
            },
            method: 'post',
            responseType: 'text',
            body: stringify(body),
        })
            .then(response => {
            const result = parse(response.body);
            if (result.error) {
                return Promise.reject(result);
            }
            delete result.status;
            delete result.error;
            return result;
        });
    }
    send(method, options) {
        const body = {
            method,
            countryCode: options.countryCode,
            mobile: options.mobile,
            smart: false,
            param1: options.param1,
            param2: options.param2,
            param3: options.param3,
            expireTime: options.expireTime ? +options.expireTime : undefined,
            templateID: options.templateID,
            code: options.code,
            length: options.length,
        };
        return this.request('/otp/send', body)
            .then(result => result.referenceID);
    }
    /**
     * Send OTP Code via SMS
     * @param options Send options
     * @returns Reference ID
     */
    sendSMS(options) {
        return this.send('sms', options);
    }
    /**
     * Send OTP Code via IVR (Interactive voice response)
     * @param options Send options
     * @returns Reference ID
     */
    sendIVR(options) {
        return this.send('ivr', options);
    }
    /**
     * Send OTP Code via Gap Messenger ([gap.im](https://gap.im))
     * @returns Reference ID
     */
    sendGapMessage(options) {
        return this.send('gap', options);
    }
    /**
     * Verify the user entered code.
     * @returns If the code is correct the promise resolves.
     */
    verify(options) {
        const body = {
            countryCode: options.countryCode,
            mobile: options.mobile,
            otp: options.otp,
        };
        return this.request('/otp/verify', body);
    }
    /**
     * Get status of the sent OTP.
     * @param referenceID
     */
    getStatus(options) {
        const body = {
            OTPReferenceID: options.OTPReferenceID,
        };
        return this.request('/otp/status', body);
    }
}
exports.GsOTP = GsOTP;
function stringifyReplacer(key, value) {
    return (typeof value === 'bigint') ? `BigInt(${String(value)})` : value;
}
function stringify(object) {
    let string = JSON.stringify(object, stringifyReplacer);
    string = string.replace(/"BigInt\((\d+)\)"/g, '$1');
    return string;
}
function jsonParseReviver(key, value) {
    if (typeof value === 'string' && /^BigInt\(\d+\)$/.test(value)) {
        return BigInt(value.match(/\d+/)[0]);
    }
    return value;
}
function parse(string) {
    string = string.replace(/\:\s*(\d{15}\d+)\s*([,}])/g, ':"BigInt($1)"$2');
    return JSON.parse(string, jsonParseReviver);
}
